/*
 * ToRegExp.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#include "ToRegExp.h"
#include "ToRegExpStateElimination.h"
#include <exception/AlibException.h>

namespace automaton {

namespace convert {

regexp::RegExp ToRegExp::convert(const automaton::Automaton& automaton) {
	regexp::RegExp* out = NULL;
	automaton.getData().Accept((void*) &out, ToRegExp::TO_REGEXP);
	regexp::RegExp res = std::move(*out);
	delete out;
	return res;
}

void ToRegExp::Visit(void* data, const automaton::EpsilonNFA& automaton) const {
	regexp::RegExp* & out = *((regexp::RegExp**) data);
	out = new regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

void ToRegExp::Visit(void* data, const automaton::MultiInitialStateNFA& automaton) const {
	regexp::RegExp* & out = *((regexp::RegExp**) data);
	out = new regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

void ToRegExp::Visit(void* data, const automaton::NFA& automaton) const {
	regexp::RegExp* & out = *((regexp::RegExp**) data);
	out = new regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

void ToRegExp::Visit(void* data, const automaton::DFA& automaton) const {
	regexp::RegExp* & out = *((regexp::RegExp**) data);
	out = new regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

void ToRegExp::Visit(void* data, const automaton::ExtendedNFA& automaton) const {
	regexp::RegExp* & out = *((regexp::RegExp**) data);
	out = new regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

void ToRegExp::Visit(void* data, const automaton::CompactNFA& automaton) const {
	regexp::RegExp* & out = *((regexp::RegExp**) data);
	out = new regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

void ToRegExp::Visit(void*, const DPDA&) const {
	throw exception::AlibException("Unsupported automaton type DPDA");
}

void ToRegExp::Visit(void*, const SinglePopDPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopDPDA");
}

void ToRegExp::Visit(void*, const InputDrivenNPDA&) const {
	throw exception::AlibException("Unsupported automaton type InputDrivenNPDA");
}

void ToRegExp::Visit(void*, const VisiblyPushdownDPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownDPDA");
}

void ToRegExp::Visit(void*, const VisiblyPushdownNPDA&) const {
	throw exception::AlibException("Unsupported automaton type VisiblyPushdownNPDA");
}

void ToRegExp::Visit(void*, const RealTimeHeightDeterministicDPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicDPDA");
}

void ToRegExp::Visit(void*, const RealTimeHeightDeterministicNPDA&) const {
	throw exception::AlibException("Unsupported automaton type RealTimeHeightDeterministicNPDA");
}

void ToRegExp::Visit(void*, const NPDA&) const {
	throw exception::AlibException("Unsupported automaton type NPDA");
}

void ToRegExp::Visit(void*, const SinglePopNPDA&) const {
	throw exception::AlibException("Unsupported automaton type SinglePopNPDA");
}

void ToRegExp::Visit(void*, const OneTapeDTM&) const {
	throw exception::AlibException("Unsupported automaton type OneTapeDTM");
}

const ToRegExp ToRegExp::TO_REGEXP;

} /* namespace convert */

} /* namespace automaton */
