/*
 * ExactMatchingAutomaton.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#include "ExactMatchingAutomaton.h"
#include <exception/AlibException.h>
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactMatchingAutomaton::construct(const string::String& pattern) {
	automaton::Automaton* out = NULL;
	pattern.getData().Accept((void*) &out, ExactMatchingAutomaton::EXACT_MATCHING_AUTOMATON);
	automaton::Automaton res = std::move(*out);
	delete out;
	return res;
}

automaton::NFA ExactMatchingAutomaton::construct(const string::LinearString& pattern) {
	automaton::NFA res(automaton::State(0));
	res.setInputSymbols(pattern.getAlphabet());
	for(const alphabet::Symbol& symbol : pattern.getAlphabet()) {
		res.addTransition(automaton::State(0), symbol, automaton::State(0));
	}
	int i = 1;
	for(const alphabet::Symbol& symbol : pattern.getContent()) {
		res.addState(automaton::State(i));
		res.addTransition(automaton::State(i-1), symbol, automaton::State(i));
		i++;
	}
	res.addFinalState(automaton::State(i-1));
	return res;
}

void ExactMatchingAutomaton::Visit(void*, const string::Epsilon&) const {
	throw exception::AlibException("Unsupported string type Epsilon");
}

void ExactMatchingAutomaton::Visit(void* data, const string::LinearString& pattern) const {
	automaton::Automaton* & out = *((automaton::Automaton**) data);
	out = new automaton::Automaton(this->construct(pattern));
}

void ExactMatchingAutomaton::Visit(void*, const string::CyclicString&) const {
	throw exception::AlibException("Unsupported string type CyclicString");
}

const ExactMatchingAutomaton ExactMatchingAutomaton::EXACT_MATCHING_AUTOMATON;

} /* namespace exact */

} /* namespace stringology */
